@@ -1,14 +1,13 @@
 #!/usr/bin/perl -d:DProf
 
 use lib 'lib', '../lib';	
-use Exception::Base 'Exception';
-
+use Exception::Base 'all';
 
 my $n = 0;
 
 foreach (1..10000) {
-    try Exception eval { throw Exception message=>'Message'; };
-    if (catch Exception my $e) {
+    try eval { throw Exception message=>'Message' };
+    if (catch my $e) {
         if ($e->isa('Exception') and $e->with('Message')) { $n++; }
     }
 }
